@@ -1,11 +1,17 @@
-// Copyright (C) 2008-2014 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup Base
@@ -18,8 +24,8 @@ struct Base_inv_yes
   {
   arma_inline const Op<derived,op_inv> i() const;   //!< matrix inverse
   
-  arma_inline const Op<derived,op_inv> i(const bool ) const;   //!< kept only for compatibility with old user code
-  arma_inline const Op<derived,op_inv> i(const char*) const;   //!< kept only for compatibility with old user code
+  arma_deprecated inline const Op<derived,op_inv> i(const bool ) const;   //!< kept only for compatibility with old user code
+  arma_deprecated inline const Op<derived,op_inv> i(const char*) const;   //!< kept only for compatibility with old user code
   };
 
 
@@ -121,6 +127,9 @@ struct Base
   
   inline elem_type min(uword& row_of_min_val, uword& col_of_min_val) const;
   inline elem_type max(uword& row_of_max_val, uword& col_of_max_val) const;
+  
+  inline arma_warn_unused uword index_min() const;
+  inline arma_warn_unused uword index_max() const;
   };
 
 
