@@ -1,18 +1,22 @@
-// Copyright (C) 2008-2016 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 
 #undef arma_hot
 #undef arma_cold
-#undef arma_pure
-#undef arma_const
 #undef arma_aligned
 #undef arma_align_mem
 #undef arma_warn_unused
@@ -24,8 +28,6 @@
 
 #define arma_hot
 #define arma_cold
-#define arma_pure
-#define arma_const
 #define arma_aligned
 #define arma_align_mem
 #define arma_warn_unused
@@ -35,7 +37,6 @@
 #define arma_noinline
 #define arma_ignore(variable)  ((void)(variable))
 
-
 #undef arma_fortran_noprefix
 #undef arma_fortran_prefix
 
@@ -65,16 +66,6 @@
 #define ARMA_INCFILE_WRAP(x) <x>
 
 
-#if defined(__CYGWIN__)
-  #if defined(ARMA_USE_CXX11)
-    #pragma message ("WARNING: Cygwin may have incomplete support for C++11 features;")
-    #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
-    #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
-    #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
-  #endif
-#endif
-
-
 #if defined(ARMA_USE_CXX11)
   
   #undef  ARMA_USE_U64S64
@@ -87,7 +78,7 @@
   
   #if defined(ARMA_64BIT_WORD) && defined(SIZE_MAX)
     #if (SIZE_MAX < 0xFFFFFFFFFFFFFFFFull)
-      #pragma message ("WARNING: disabled use of 64 bit integers, as std::size_t is smaller than 64 bits")
+      // #pragma message ("WARNING: disabled use of 64 bit integers, as std::size_t is smaller than 64 bits")
       #undef ARMA_64BIT_WORD
     #endif
   #endif
@@ -132,14 +123,17 @@
 #endif
 
 
-#if defined(__APPLE__)
+#if defined(__APPLE__) || defined(__apple_build_version__)
   #undef  ARMA_BLAS_SDOT_BUG
   #define ARMA_BLAS_SDOT_BUG
+  
   #undef  ARMA_HAVE_POSIX_MEMALIGN
+  #undef  ARMA_USE_EXTERN_CXX11_RNG
+  // TODO: thread local storage (TLS) (eg. "extern thread_local") appears currently broken on Mac OS X
 #endif
 
 
-#if defined(__MINGW32__)
+#if defined(__MINGW32__) || defined(__CYGWIN__) || defined(_MSC_VER)
   #undef ARMA_HAVE_POSIX_MEMALIGN
 #endif
 
@@ -159,7 +153,7 @@
 #endif
 
 
-#if (defined(__GNUG__) || defined(__GNUC__)) && (defined(__clang__) || defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__))
+#if (defined(__GNUG__) || defined(__GNUC__)) && (defined(__clang__) || defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__) || defined(__ARMCC_VERSION) || defined(__IBMCPP__))
   #undef  ARMA_FAKE_GCC
   #define ARMA_FAKE_GCC
 #endif
@@ -170,10 +164,15 @@
   #undef  ARMA_GCC_VERSION
   #define ARMA_GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
   
-  #if (ARMA_GCC_VERSION < 40200)
+  #if (ARMA_GCC_VERSION < 40400)
     #error "*** Need a newer compiler ***"
   #endif
   
+  #if (ARMA_GCC_VERSION < 40600)
+    #undef  ARMA_PRINT_CXX98_WARNING
+    #define ARMA_PRINT_CXX98_WARNING
+  #endif
+  
   #if ( (ARMA_GCC_VERSION >= 40700) && (ARMA_GCC_VERSION <= 40701) )
     #error "gcc versions 4.7.0 and 4.7.1 are unsupported; use 4.7.2 or later"
     // due to http://gcc.gnu.org/bugzilla/show_bug.cgi?id=53549
@@ -181,8 +180,8 @@
   
   #define ARMA_GOOD_COMPILER
   
-  #undef  arma_pure
-  #undef  arma_const
+  #undef  arma_hot
+  #undef  arma_cold
   #undef  arma_aligned
   #undef  arma_align_mem
   #undef  arma_warn_unused
@@ -191,8 +190,8 @@
   #undef  arma_inline
   #undef  arma_noinline
   
-  #define arma_pure               __attribute__((__pure__))
-  #define arma_const              __attribute__((__const__))
+  #define arma_hot                __attribute__((__hot__))
+  #define arma_cold               __attribute__((__cold__))
   #define arma_aligned            __attribute__((__aligned__))
   #define arma_align_mem          __attribute__((__aligned__(16)))
   #define arma_warn_unused        __attribute__((__warn_unused_result__))
@@ -206,28 +205,17 @@
   
   #if defined(ARMA_USE_CXX11)
     #if (ARMA_GCC_VERSION < 40800)
-      #pragma message ("WARNING: compiler is in C++11 mode, but it has incomplete support for C++11 features;")
-      #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
-      #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
-      #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
-      #define ARMA_DONT_USE_CXX11_CHRONO
+      #undef  ARMA_PRINT_CXX11_WARNING
+      #define ARMA_PRINT_CXX11_WARNING
     #endif
   #endif
   
-  #if !defined(ARMA_USE_CXX11)
+  #if !defined(ARMA_USE_CXX11) && !defined(__GXX_EXPERIMENTAL_CXX0X__) && (__cplusplus < 201103L) 
     #if defined(_GLIBCXX_USE_C99_MATH_TR1) && defined(_GLIBCXX_USE_C99_COMPLEX_TR1)
       #define ARMA_HAVE_TR1
     #endif
   #endif
   
-  #if (ARMA_GCC_VERSION >= 40300)
-    #undef  arma_hot
-    #undef  arma_cold
-    
-    #define arma_hot  __attribute__((__hot__))
-    #define arma_cold __attribute__((__cold__))
-  #endif
-  
   #if (ARMA_GCC_VERSION >= 40700)
     #define ARMA_HAVE_GCC_ASSUME_ALIGNED
   #endif
@@ -239,7 +227,7 @@
     #define ARMA_SIMPLE_LOOPS
   #endif
   
-  #if (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
+  #if !defined(ARMA_USE_CXX11) && (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
     #define ARMA_HAVE_SNPRINTF
     #define ARMA_HAVE_ISFINITE
     #define ARMA_HAVE_LOG1P
@@ -247,12 +235,10 @@
     #define ARMA_HAVE_ISNAN
   #endif
   
-  #undef ARMA_GCC_VERSION
-  
 #endif
 
 
-#if defined(__clang__) && (defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__))
+#if defined(__clang__) && (defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__) || defined(__ARMCC_VERSION) || defined(__IBMCPP__))
   #undef  ARMA_FAKE_CLANG
   #define ARMA_FAKE_CLANG
 #endif
@@ -266,16 +252,6 @@
     #define __has_attribute(x) 0
   #endif
   
-  #if __has_attribute(__pure__)
-    #undef  arma_pure
-    #define arma_pure __attribute__((__pure__))
-  #endif
-  
-  #if __has_attribute(__const__)
-    #undef  arma_const
-    #define arma_const __attribute__((__const__))
-  #endif
-  
   #if __has_attribute(__aligned__)
     #undef  arma_aligned
     #undef  arma_align_mem
@@ -327,12 +303,7 @@
     #define ARMA_HAVE_GCC_ASSUME_ALIGNED
   #endif
   
-  #if defined(__apple_build_version__)
-    #undef ARMA_USE_EXTERN_CXX11_RNG
-    // TODO: check the status of support for "extern thread_local" in clang shipped with Mac OS X
-  #endif
-  
-  #if (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
+  #if !defined(ARMA_USE_CXX11) && (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
     #define ARMA_HAVE_SNPRINTF
     #define ARMA_HAVE_ISFINITE
     #define ARMA_HAVE_LOG1P
@@ -345,7 +316,11 @@
 
 #if defined(__INTEL_COMPILER)
   
-  #if (__INTEL_COMPILER_BUILD_DATE < 20090623)
+  #if (__INTEL_COMPILER == 9999)
+    #error "*** Need a newer compiler ***"
+  #endif
+  
+  #if (__INTEL_COMPILER < 1300)
     #error "*** Need a newer compiler ***"
   #endif
   
@@ -353,36 +328,44 @@
   #undef  ARMA_HAVE_ICC_ASSUME_ALIGNED
   #define ARMA_HAVE_ICC_ASSUME_ALIGNED
   
+  #if defined(ARMA_USE_CXX11)
+    #if (__INTEL_COMPILER < 1500)
+      #undef  ARMA_PRINT_CXX11_WARNING
+      #define ARMA_PRINT_CXX11_WARNING
+    #endif
+  #endif
+  
 #endif
 
 
 #if defined(_MSC_VER)
   
-  #if (_MSC_VER < 1600)
+  #if (_MSC_VER < 1700)
     #error "*** Need a newer compiler ***"
   #endif
   
-  #if (_MSC_VER < 1700)
-    #pragma message ("WARNING: this compiler is outdated and has incomplete support for the C++ standard;")
-    #pragma message ("WARNING: if something breaks, you get to keep all the pieces")
-    #define ARMA_BAD_COMPILER
+  #if (_MSC_VER < 1800)
+    #undef  ARMA_PRINT_CXX98_WARNING
+    #define ARMA_PRINT_CXX98_WARNING
   #endif
   
   #if defined(ARMA_USE_CXX11)
     #if (_MSC_VER < 1900)
-      #pragma message ("WARNING: compiler is in C++11 mode, but it has incomplete support for C++11 features;")
-      #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
-      #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
-      #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
+      #undef  ARMA_PRINT_CXX11_WARNING
+      #define ARMA_PRINT_CXX11_WARNING
     #endif
   #endif
   
+  #undef  arma_deprecated
+  #define arma_deprecated __declspec(deprecated)
   // #undef  arma_inline
   // #define arma_inline inline __forceinline
   
   #pragma warning(push)
   
   #pragma warning(disable: 4127)  // conditional expression is constant
+  #pragma warning(disable: 4180)  // qualifier has no meaning
+  #pragma warning(disable: 4244)  // possible loss of data when converting types
   #pragma warning(disable: 4510)  // default constructor could not be generated
   #pragma warning(disable: 4511)  // copy constructor can't be generated
   #pragma warning(disable: 4512)  // assignment operator can't be generated
@@ -397,6 +380,7 @@
   #pragma warning(disable: 4710)  // function not inlined
   #pragma warning(disable: 4711)  // call was inlined
   #pragma warning(disable: 4714)  // __forceinline can't be inlined
+  #pragma warning(disable: 4800)  // value forced to bool
   
   // #if (_MANAGED == 1) || (_M_CEE == 1)
   //   
@@ -431,10 +415,105 @@
   #if (__SUNPRO_CC < 0x5100)
     #error "*** Need a newer compiler ***"
   #endif
-    
+  
+  #if defined(ARMA_USE_CXX11)
+    #if (__SUNPRO_CC < 0x5130)
+      #undef  ARMA_PRINT_CXX11_WARNING
+      #define ARMA_PRINT_CXX11_WARNING
+    #endif
+  #endif
+  
 #endif
 
 
+#if defined(ARMA_USE_CXX11) && defined(__CYGWIN__) && !defined(ARMA_DONT_PRINT_CXX11_WARNING)
+  #pragma message ("WARNING: Cygwin may have incomplete support for C++11 features.")
+#endif
+
+
+#if defined(ARMA_USE_CXX11) && (__cplusplus < 201103L)
+  #undef  ARMA_PRINT_CXX11_WARNING
+  #define ARMA_PRINT_CXX11_WARNING
+#endif
+
+
+#if defined(ARMA_PRINT_CXX98_WARNING) && !defined(ARMA_DONT_PRINT_CXX98_WARNING)
+  #pragma message ("WARNING: this compiler is OUTDATED and has INCOMPLETE support for the C++ standard;")
+  #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
+#endif
+
+
+#if defined(ARMA_PRINT_CXX11_WARNING) && !defined(ARMA_DONT_PRINT_CXX11_WARNING)
+  #pragma message ("WARNING: use of C++11 features has been enabled,")
+  #pragma message ("WARNING: but this compiler has INCOMPLETE support for C++11;")
+  #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
+  #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
+  #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
+#endif
+
+
+#if ( defined(ARMA_USE_OPENMP) && (!defined(_OPENMP) || (defined(_OPENMP) && (_OPENMP < 201107))) )
+  // we require OpenMP 3.0 to enable parallelisation of for loops with unsigned integers;
+  // earlier versions of OpenMP can only handle signed integers;
+  // we require OpenMP 3.1 for atomic read and atomic write
+  #undef  ARMA_USE_OPENMP
+  #undef  ARMA_PRINT_OPENMP_WARNING
+  #define ARMA_PRINT_OPENMP_WARNING
+#endif
+
+
+#if ( (defined(_OPENMP) && (_OPENMP < 201107)) && !defined(ARMA_DONT_USE_OPENMP) )
+  // if the compiler has an ancient version of OpenMP and use of OpenMP hasn't been explicitly disabled,
+  // print a warning to ensure there is no confusion about OpenMP support
+  #undef  ARMA_USE_OPENMP
+  #undef  ARMA_PRINT_OPENMP_WARNING
+  #define ARMA_PRINT_OPENMP_WARNING
+#endif
+
+
+#if defined(ARMA_PRINT_OPENMP_WARNING) && !defined(ARMA_DONT_PRINT_OPENMP_WARNING)
+  #pragma message ("WARNING: use of OpenMP disabled; compiler support for OpenMP 3.1+ not detected")
+#endif
+
+
+#if defined(ARMA_USE_OPENMP) && !defined(ARMA_USE_CXX11)
+  #if (defined(ARMA_GCC_VERSION) && (ARMA_GCC_VERSION >= 50400)) || (defined(__clang__) && !defined(ARMA_FAKE_CLANG))
+    #undef  ARMA_PRINT_OPENMP_CXX11_WARNING
+    #define ARMA_PRINT_OPENMP_CXX11_WARNING
+  #endif
+#endif
+
+
+#if defined(ARMA_PRINT_OPENMP_CXX11_WARNING) && !defined(ARMA_DONT_PRINT_OPENMP_WARNING)
+  #pragma message ("WARNING: support for OpenMP requires C++11/C++14; add -std=c++11 or -std=c++14 to compiler flags")
+#endif
+
+
+
+#if defined(ARMA_USE_OPENMP) && defined(ARMA_USE_CXX11)
+  #if (defined(ARMA_GCC_VERSION) && (ARMA_GCC_VERSION < 50400))
+    // due to https://gcc.gnu.org/bugzilla/show_bug.cgi?id=57580
+    #undef ARMA_USE_OPENMP
+    #if !defined(ARMA_DONT_PRINT_OPENMP_WARNING)
+      #pragma message ("WARNING: use of OpenMP disabled due to compiler bug in gcc <= 5.3")
+    #endif
+  #endif
+#endif
+
+
+
+// cleanup
+
+#undef ARMA_FAKE_GCC
+#undef ARMA_FAKE_CLANG
+#undef ARMA_GCC_VERSION
+#undef ARMA_PRINT_CXX98_WARNING
+#undef ARMA_PRINT_CXX11_WARNING
+#undef ARMA_PRINT_OPENMP_WARNING
+#undef ARMA_PRINT_OPENMP_CXX11_WARNING
+
+
+
 #if defined(log2)
   #undef log2
   #pragma message ("WARNING: detected 'log2' macro and undefined it")
