@@ -1,11 +1,17 @@
-// Copyright (C) 2008-2015 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup fn_det
@@ -14,8 +20,8 @@
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, typename T1::elem_type >::result
 det
   (
@@ -30,8 +36,8 @@ det
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename T1::elem_type
 det
   (
@@ -70,8 +76,8 @@ det
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename T1::elem_type
 det
   (
@@ -110,8 +116,8 @@ det
 
 //! determinant of inv(A), without doing the inverse operation
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, typename T1::elem_type >::result
 det
   (
@@ -131,9 +137,10 @@ det
 
 
 
+//! NOTE: don't use this form: it will be removed
 template<typename T1>
+arma_deprecated
 inline
-arma_warn_unused
 typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, typename T1::elem_type >::result
 det
   (
@@ -143,14 +150,17 @@ det
   {
   arma_extra_debug_sigprint();
   
+  // arma_debug_warn("det(X,bool) is deprecated and will be removed; change to det(X)");
+  
   return det(X.get_ref());
   }
 
 
 
+//! NOTE: don't use this form: it will be removed
 template<typename T1>
+arma_deprecated
 inline
-arma_warn_unused
 typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, typename T1::elem_type >::result
 det
   (
@@ -160,14 +170,16 @@ det
   {
   arma_extra_debug_sigprint();
   
+  // arma_debug_warn("det(X,char*) is deprecated and will be removed; change to det(X)");
+  
   return det(X.get_ref());
   }
 
 
 
 template<typename T>
-arma_inline
 arma_warn_unused
+arma_inline
 const typename arma_scalar_only<T>::result &
 det(const T& x)
   {
