@@ -1,9 +1,17 @@
-// Copyright (C) 2013 Conrad Sanderson
-// Copyright (C) 2013 NICTA (www.nicta.com.au)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
+// 
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 
@@ -22,6 +30,16 @@ class op_any
   any_vec_helper(const Base<typename T1::elem_type, T1>& X);
   
   
+  template<typename eT>
+  static inline bool
+  any_vec_helper(const subview<eT>& X);
+  
+  
+  template<typename T1>
+  static inline bool
+  any_vec_helper(const Op<T1, op_vectorise_col>& X);
+  
+  
   template<typename T1, typename op_type>
   static inline bool
   any_vec_helper
