@@ -1,11 +1,17 @@
-// Copyright (C) 2008-2014 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup operator_ostream
@@ -61,6 +67,48 @@ operator<< (std::ostream& o, const SpValProxy<T1>& X)
 
 
 
+template<typename eT>
+inline
+std::ostream&
+operator<< (std::ostream& o, const MapMat_val<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  o << eT(X);
+  
+  return o;
+  }
+
+
+
+template<typename eT>
+inline
+std::ostream&
+operator<< (std::ostream& o, const MapMat_elem<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  o << eT(X);
+  
+  return o;
+  }
+
+
+
+template<typename eT>
+inline
+std::ostream&
+operator<< (std::ostream& o, const MapMat_svel<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  o << eT(X);
+  
+  return o;
+  }
+
+
+
 template<typename T1>
 inline
 std::ostream&
