@@ -295,3 +295,9 @@ arma::umat r_umat_test(arma::umat& v) { return(v); }
 
 // [[Rcpp::export]]
 arma::umat cr_umat_test(const arma::umat& v) { return(v); }
+
+// [[Rcpp::export]]
+arma::vec vecc_test(arma::vec v) { return(v); }
+
+// [[Rcpp::export]]
+arma::rowvec vecr_test(arma::rowvec v) { return(v); }
